/*****************************************************************************
 * 
 * NAME: BuildParmList.c
 *
 * DESCRIPTION: This function build a list for all aggragates of Label and 
 *     of their Aggergate in the .met file. 
 *     
 *
 * PARAMETERS:
 *     pRootF1 (AGGREGATE)     pointer to the first odl label root (.met file).
 *     
 *     pRootF2 (AGGREGATE)     pointer to the second odl label root (.desc file).
 *    
 *     OUTput (FILE)           pointer to the out put file.
 *
 * RETURN VALUE:
 *     
 *          
 * GLOBAL VARIABLES:
 *     None.
 *	
 * PRE- & POST-CONDITIONS:
 * 
 *     1. PRE:  If .desc Agg is mandatory and was not found in the .met file
 *        POST: Error message generated stating such.  The agg was not found in the .met file
 *             
 *     2. PRE:  If .desc Agg is optional and was not found in the .met file
 *        POST: Warning message generated stating such.  The agg was not found in the .met file
 *              
 *
 * NOTES:
 *
 *
 *
 * REVISION HISTORY:
 *     07/26/00  Amer Rahabi
 *
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "lstList.h"
#include "strUtilities.h"
#include "odlFieldList.h"



lstLIST buildFieldList(AGGREGATE mainAgg)
{

  AGGREGATE mainSibAgg = NULL;
  AGGREGATE stopAgg = NULL;
  AGGREGATE curAgg = NULL;
  AGGREGATE tmpAgg = NULL;
  PARAMETER pParm;
  VALUE     pValue;

  char pathName[2000],aggType[100],fieldName[200];

  char man[10],repeat[10];  

  descList fldEntry;
  lstLIST fldList = NULL;

  fldList = lstInitList();

  stopAgg = mainAgg->parent;

 
  /*   for each sibling find all field name descriptions */

   mainSibAgg = NextAggregate(mainAgg);
    while(mainSibAgg) {
      curAgg = NextAggregate(mainSibAgg);
      while(curAgg) { 
	if(fieldDesc(curAgg)){	/* we have a field descriptor */  
            strcpy(man,"FALSE");
            pParm = FindParameter(curAgg,"MANDATORY");
            if(pParm) {
              pValue = FirstValue(pParm);
              strcpy(man,pValue->item.value.string);
	    }
      
          
           strcpy(repeat,"NO");
            pParm = FindParameter(curAgg->parent,"CLASS");
            if(pParm) {
              pValue = FirstValue(pParm);
              strcpy(repeat,pValue->item.value.string);
	    }

            /*pAgg = curAgg; */
            strcpy(fieldName,curAgg->name);
            buildOdlPath(curAgg,stopAgg,pathName,aggType); 

            fldEntry = malloc(sizeof(struct descList));

            fldEntry->fieldName = malloc(strlen(fieldName)+1);     
            strcpy(fldEntry->fieldName,fieldName);

            strcpy(fldEntry->fieldMandatory,man);

            strcpy(fldEntry->fieldRepeat,repeat);

            fldEntry->fieldPath = malloc(strlen(pathName)+1);     
            strcpy(fldEntry->fieldPath,pathName);
          

            fldEntry->pathType = malloc(strlen(aggType)+1);     
            strcpy(fldEntry->pathType,aggType);
        
            lstAddLast(fldList, fldEntry);


        }

        curAgg = NextAggregate(curAgg);
        if ( mainSibAgg->right_sibling == curAgg)                 
        	  curAgg = NULL;
        else if(curAgg->parent == stopAgg)
          curAgg = NULL; 
      }/* end while curAgg */
  
      mainSibAgg = mainSibAgg->right_sibling; /* Process next sibling */ 

    } /*end  while(mainSibAgg) */

	return fldList;

}

int fieldDesc(AGGREGATE agg)
{
  
       if(!FindParameter(agg,"TYPE"))
             return FALSE;

       if(!FindParameter(agg,"DATA_LOCATION"))
		     return FALSE;
	   if(!FindParameter(agg,"MANDATORY"))
		     return FALSE;
	   if(FindParameter(agg,"NUM_VAL"))
		     return TRUE;
	   if(FindParameter(agg,"MAXOCCURRENCES"))
		     return TRUE;

       return FALSE;
}

void buildOdlPath(AGGREGATE start, AGGREGATE stop,char *pathName,
                  char *aggType)

{
   AGGREGATE nextS;
   char tmpName[200];
   char tmpType[200];
   char *r;
   int i;
   nextS = start;
  
   pathName[0] = '\0';
   tmpType[0] = '\0';
   while(nextS->parent != stop) {
     sprintf(tmpType,"%s%d",tmpType,nextS->kind);
      strcpy(tmpName,pathName);
      strcpy(pathName,nextS->name);
      if(strlen(tmpName) >0){
        strcat(pathName,"/");
        strcat(pathName,tmpName);
      }
      nextS = nextS->parent;
    
   }

    r = tmpType + strlen(tmpType)-1;  
    for(i = 0;r >=  tmpType; r--) 
        aggType[i++] = *r;
     aggType[strlen(tmpType)] = '\0';     
 
}

void printFldList(lstLIST fldList)


{

 descList descEntry;
 lstStopIteration(fldList);
 descEntry = (descList) lstIterator(fldList);
 while (descEntry != NULL){
    printf("Name = %s Man = %s Rep = %s path =%s\n",
      descEntry->fieldName,descEntry->fieldRepeat,
      descEntry->fieldMandatory,descEntry->pathType);
    printf("Path = %s\n\n",descEntry->fieldPath);

    descEntry = (descList) lstIterator(fldList);
    

  }   

}

descList getFldEntry(int first,lstLIST flist)
{
if (first)
 lstStopIteration(flist);

 return lstIterator(flist);
}

AGGREGATE getAgg(AGGREGATE startAgg,char *fieldPath, char *pathType) 
{

AGGREGATE tmpAgg;
AGGREGATE baseAgg;

int i;
char *aggName;
char fTok[] = "/";


if (strlen(fieldPath) == 0)
       return NULL;
   baseAgg = startAgg;
   
   for (aggName = strNextToken (fieldPath, fTok), i = 1;
       aggName != NULL;
       aggName = strNextToken (NULL,fTok), i++) {

	if(pathType[i-1] == '0')
          tmpAgg = FindObject(baseAgg,aggName,NULL);
       else
          tmpAgg = FindGroup(baseAgg,aggName,NULL);

         if (tmpAgg == NULL) {
             printf("Aggregate Not Found - %s\n",aggName);
             return NULL;
		 }
       
         if(tmpAgg->parent != baseAgg) {
             printf(" Aggregate Path does not Match - %s\n",aggName);
			 return NULL;
	  
        }
       baseAgg = tmpAgg;
       }

 return tmpAgg;

  }

       



