/*****************************************************************************
 * 
 * NAME: ManObjCheck.c
 *
 * DESCRIPTION: This function check the mandatory fields for PSA's and Containers 
 *     
 *     
 *
 * PARAMETERS:
 *     pRootF1 (AGGREGATE)     pointer to the first odl label root (.met file).
 *     
 *     pRootF2 (AGGREGATE)     pointer to the second odl label root (.desc file).
 *    
 *     OUTput (FILE)           pointer to the out put file.
 *
 * RETURN VALUE:
 *     
 *          
 * GLOBAL VARIABLES:
 *     None.
 *	
 * PRE- & POST-CONDITIONS:
 * 
 *     1. PRE:  If .desc Agg is mandatory and was not found in the .met file
 *        POST: Error message generated stating such.  The agg was not found in the .met file
 *             
 *     2. PRE:  If .desc Agg is optional and was not found in the .met file
 *        POST: Warning message generated stating such.  The agg was not found in the .met file
 *              
 *
 * NOTES:
 *
 *
 *
 * REVISION HISTORY:
 *     07/26/00  Amer Rahabi
 *
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "metcheck.h"
#include "odldef.h"
#include "odlinter.h"
#include "lstList.h"
#include "odlFieldList.h"

int ManObjCheck(AGGREGATE  pRootF1,AGGREGATE  pRootF2, FILE *OUTput)
{
 
  AGGREGATE invGroup, descGroup = NULL;
 /* AGGREGATE tmpAgg; */
  lstLIST fldList = NULL;
  lstLIST grpList = NULL;

 /* descList fldEntry; */
  int Failure = 0;
  int Success = 0;
  int pass =1;

 
/* initilaizing to the begining of .met file */

  invGroup = FindGroup(pRootF1,"INVENTORYMETADATA");
  if(invGroup == NULL)
  {
    pass =0;
    checkFatalError ( "The .met file",
                   "the tool unable to find the main aggregate INVENTORYMETADATA" );
    
  }

  /* initilaizing to the begining of .desc file */

  descGroup = FindGroup(pRootF2,"INVENTORYMETADATA");
  if(invGroup == NULL)
  {
    pass =0;
    checkFatalError ( "The .desc file",
                   "the tool unable to find the main aggregate INVENTORYMETADATA" );
    
  }

/* build the structure to hold PSA's and Containers  for further checking */
    
    
  grpList = lstInitList();

  if ( ! buildFieldList(descGroup, grpList ) )
  {
    pass =0;
    checkFatalError ( "The .met file",
                              "No Field Descriptions were located in the discriptor file" ); 
  }

/* check mandatory objects for PSA's and Containers . */

  if ( !checkMetGroups(grpList, invGroup, descGroup,OUTput) )
  {
     checkWarnings( "checkMetGroups",
                   "Warning  :: some errors were found while runnig " );
     pass =0;
  }
/* return status to the calling function for debug porposes */

 if( pass == 0)
   return Failure;
 else
  return Success;

  }


