/* FILE: CheckMandatoryMet.c
 *
 * PROGRAM: Check for Mandatory Fields
 *
 * DESCRIPTION - These functions checks that mandatory field Object are
 *               present in the MET file.
 *
 * checkMetGroups - checks that each required group is contained in the
 *                  MET file and calls functions to check individual data
 *                  Objects.
 *
 * CheckRepeatGroups - Gets repeatable aggregates and calls function to
 *                     check individual data objects.
 *
 * checkMetFields - Checks for mandatory data objects.
 *
 * checkMandatoryPSA - Checks for mandatory PSAs.
 *
 * addAttributeExists - searchs the MET file for a specfic PSA.
 *
 * getAgg - Returns a pointer to a specfic aggregate.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "odldef.h"
#include "odlinter.h"
#include "lstList.h"
#include "strUtilities.h"
#include "odlFieldList.h"
#include "checkMandatoryMet.h"
#include "checkI.h"
#include "metcheck.h"

/* Internal fuction prototypes */
int checkMandatoryPSA(lstLIST fldList, AGGREGATE startAgg, FILE *OUTput);
int addAttributeExists(AGGREGATE startAgg, char *fieldName);
void checkMetFields(lstLIST fldList,AGGREGATE grpAgg,char * Cclass,FILE *OUTput);
void checkRepeatGroups(groupList grpEntry,AGGREGATE grpAgg,FILE *Output);
  /* external varibles */
extern metArgStructure CONTROLS;     /* metcheck controls */
/*******************************/

int manMissing; /*counter for missing mandatory fields */
/****************************************************************/
int checkMetGroups(lstLIST grpList,AGGREGATE mainMetAgg, AGGREGATE mainDescAgg,
				   FILE *OUTput)
{

  lstLIST fldList;	
  descList fldEntry = NULL;
  groupList  grpEntry = NULL;
  AGGREGATE grpAgg;
 
 
  manMissing = 0;
  lstStopIteration(grpList); 
  grpEntry = (groupList) lstIterator(grpList);
  while (grpEntry != NULL){  /* loop to process each INVENTORYMetaData Sibbling*/
      fldList = grpEntry->groupFields;

      if(strcmp(grpEntry->groupPath,"PRODUCTSPECIFICMETADATA") == 0) {
		  checkMandatoryPSA(fldList,mainMetAgg,CONTROLS.pOutputFile);
      }

      else { /*Not a PSA Group. Get the Group from the MET file. */
		  grpAgg = getAgg(mainMetAgg,grpEntry->groupPath,grpEntry->groupType);
          if(grpAgg) {
			  if(grpEntry->groupRepeat) /* is this a container */
				  checkRepeatGroups(grpEntry,grpAgg,OUTput);
			  else /* check all data objects within this Group */
			      checkMetFields(grpEntry->groupFields,grpAgg,NULL,OUTput);
		  }

		  else if(grpEntry->groupMandatory) { /* Group is missing from the MET file */
              fprintf (OUTput,"ERROR  :: Mandatory Group %s is missing from the Met File\n",grpEntry->groupPath);
              ++manMissing;		  
		  }
	  }
	  
  grpEntry = (groupList) lstIterator(grpList); /* get the next group pointer */
  }
return 1;
}	  
/******************************************************************
*
*  Check repeating groups
*/

void checkRepeatGroups(groupList grpEntry,AGGREGATE grpAgg, FILE *OUTput)
{
  PARAMETER pParm;
  VALUE pValue;
  int curClass;
  char classVal[10];
  curClass = 1;
  sprintf(classVal,"%d",curClass);
  pParm = FindParameter(grpAgg,"CLASS"); /* get the Container CLASS PARAMETER */
  if(pParm) {
     pValue = FirstValue(pParm); /* CLASS should equal 1.*/
     if(strcmp(pValue->item.value.string,classVal) !=0) 
        fprintf (OUTput,"ERROR  :: Invalid CLASS value for %s\n",grpEntry->groupPath);

  }
  else { /*CLASS PARAMETER is Missing */
       fprintf (OUTput,"ERROR  :: CLASS parameter missing for %s\n",grpEntry->groupPath);

	  
  }

  while(grpAgg != NULL) { /* Loop to check each container */
	 checkMetFields(grpEntry->groupFields,grpAgg,classVal,OUTput);
	 if(grpAgg->kind == KA_OBJECT)
		 grpAgg = FindNextObject(grpAgg->parent,grpAgg,grpAgg->name,NULL);
	  else
		  grpAgg = FindNextGroup(grpAgg->parent,grpAgg,grpAgg->name);
	  sprintf(classVal,"%d",++curClass);
  }

}
/**********************************************************/
void checkMetFields(lstLIST fldList,AGGREGATE metAgg,
					char *classVal, FILE *OUTput)

{
   descList fldEntry;
   AGGREGATE metField;

   char curParent[200];
   PARAMETER pParm;
   VALUE pValue;
   
   lstStopIteration(fldList);  
   strcpy(curParent,metAgg->name);
   fldEntry = getFldEntry(TRUE, fldList);
   while(fldEntry) {

	  /* Get the Data OBJECT */
	  metField = getAgg(metAgg,fldEntry->fieldPath,fldEntry->pathType);
      if(metField != NULL) {

	     if(fldEntry->fieldRepeat && strcmp(curParent,(metField->parent)->name) != 0) {
		     strcpy(curParent,(metField->parent)->name);
		     pParm = FindParameter(metField->parent,"CLASS");
 
			 if(pParm) {
                pValue = FirstValue(pParm);
                if(strcmp(pValue->item.value.string,classVal) !=0)

				  fprintf (OUTput,"ERROR  :: Invalid CLASS value for %s\n",curParent);
					
			 }
             else {
                fprintf (OUTput,"ERROR  :: CLASS parameter missing for %s\n",curParent);

		
			 }

		 }
	  }
		   
	  else if(fldEntry->fieldMandatory) { /* missing field */
      fprintf (OUTput,"ERROR  :: Mandatory Field %s Missing from the Met File\n",fldEntry->fieldName);
      ++manMissing;	  		  
	  }	    
	    
      fldEntry = getFldEntry(FALSE, fldList);
   } /* While */

	 
}    


/************************************************************/

int addAttributeExists(AGGREGATE psaAgg, char *fieldName)
 {
  
  
   AGGREGATE attNameAgg,tmpGroup = NULL;
   VALUE pValue = NULL;
   
   if(psaAgg == NULL) 
	   return FALSE;
   

   attNameAgg = FindObject(psaAgg,"ADDITIONALATTRIBUTENAME",NULL);
   while (attNameAgg != NULL) {
	   pValue = FirstValue(FindParameter(attNameAgg,"VALUE"));
       if(pValue) {
	      if(strToUpStrcmp(pValue->item.value.string,fieldName) == 0)
		     return TRUE;
	   }
	   attNameAgg= FindNextObject(psaAgg,attNameAgg,"ADDITIONALATTRIBUTENAME",NULL);
   }

   
   return FALSE;
   

 

}

/*****************************************************************/

descList getFldEntry(int first,lstLIST flist)
{
if (first)
 lstStopIteration(flist);

 return lstIterator(flist);
}
/******************************************************************/
AGGREGATE getAgg(AGGREGATE startAgg,char *fieldPath, char *pathType) 
{

  AGGREGATE tmpAgg;
  AGGREGATE baseAgg;

  int i;
  char *aggName;
  char fTok[] = "/";

  if (strlen(fieldPath) == 0)
      return NULL;

  baseAgg = startAgg;
  for (aggName = strNextToken (fieldPath, fTok), i = 1;
       aggName != NULL;
       aggName = strNextToken (NULL,fTok), i++) {
    
	if(pathType[i-1] == '0')
        tmpAgg = FindObject(baseAgg,aggName,NULL);
    else

        tmpAgg = FindGroup(baseAgg,aggName);

     if(tmpAgg == NULL) {
   
         return NULL;
	 }
       
     if(tmpAgg->parent != baseAgg) {
  
		 return NULL;
	  
     }
     baseAgg = tmpAgg;
	   
	   } /* End for Loop */

  return tmpAgg;

}


int checkMandatoryPSA(lstLIST fldList, AGGREGATE startAgg, FILE *OUTput)
{
  descList fldEntry;
 

  VALUE pValue = NULL;
  PARAMETER numValParm = NULL;
  AGGREGATE psaAgg;
	        
 
  fldEntry = getFldEntry(TRUE, fldList);
  psaAgg = FindGroup(startAgg,"ADDITIONALATTRIBUTES");

  if(psaAgg == NULL) {
 
    fprintf (OUTput,"ERROR  ::  Additional Attributes Group Missing from MET file");
    
      return 0;
  }


  while(fldEntry) 
  {
	   if(fldEntry->fieldMandatory) 
     {
		   if(!addAttributeExists(psaAgg, fldEntry->fieldName)) 
       {


        fprintf (OUTput,"ERROR  :: Mandatory PSA %s is missing from the Met File\n",fldEntry->fieldName);
        ++manMissing;
           }
	   }
	   fldEntry = getFldEntry(FALSE, fldList);
   }

return 1;
}