/*****************************************************************************
/* FILE: checkUilities.c
 *
 * PROGRAM: Utility module
 *
 * DESCRIPTION: Utility functions to check valids values.
 *
 * RCS INFO:
 *
 * NOTES:
 *
 *
 *
 * REVISION HISTORY:
 *     08/02/00  Amer Rahabi (original)
 *
 ****************************************************************************/

/* INCLUDED FILES:
*/

#include <stdio.h>

#include <stdlib.h>
#include <ctype.h>
#include "odldef.h"
#include "odlinter.h"
#include "metcheck.h"
#include "CheckI.h" 
#include "strUtilities.h"

/* Return status to return to the calling function. */
const int SUCCESS = 1;
const int FAILURE = 0; 

/*      long NumVal; */

extern long NumVal;


/*****************************************************************************
 * 
 * NAME: CheckNumVal
 *
 * DESCRIPTION: checks the NumVal parameter
 *
 * PARAMETERS:  
 *     MyAgg -- (AGGREGATE *) pointer to 1st Aggregate to be checked.
 *     RefAgg -- (AGGREGATE *) pointer to the referance Aggregate to be checked against.
 *     output    (FILE *)      pointer to the output file.                 
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the parameter passed.
 *          FAILURE (int)   the parameter failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/03/00  Amer Rahabi  (original)           
 *
 ****************************************************************************/

int CheckNumVal(AGGREGATE MyAgg, AGGREGATE  RefAgg, FILE *output)
{
/* local Variabels */

 VALUE  refParVal, myParVal= NULL;
 PARAMETER refParmNV, myParmNV = NULL;
 PARAMETER myParmV = NULL;
 int pass = 0;
 /* */
       refParmNV= FindParameter (RefAgg,"NUM_VAL");
	   if (refParmNV == NULL)
		   refParmNV = FindParameter ( RefAgg, "MAXOCCURRENCES");
	   if( refParmNV != NULL )
		{
            refParVal = FirstValue(refParmNV);
/*            strcpy(stringParVal,refParVal->item.value.string);
  */           
            NumVal = refParVal->item.value.integer.number;
            myParmNV= FindParameter (MyAgg,"NUM_VAL");
			if (!(myParmNV==NULL))
            myParVal= FirstValue(myParmNV);

			myParmV= FindParameter (MyAgg,"VALUE");
      if (myParmV == NULL){
	    fprintf (output,"ERROR :: The Aggeragate  = %s  have the parmeter VALUE missing\n",RefAgg->name);
      fprintf (output,"         it should have at least one Value \n");
       
			  pass = 0;	
      }
      else if( ((myParmNV != NULL) && (myParVal->item.value.integer.number != NumVal)) ||
                (myParmV->value_count > NumVal) )
                /* if all the above checking fails produce an error    */
            {
			      fprintf (output,"ERROR :: The Aggeragate  = %s  have the parmeter NUM_VAl missing\n",RefAgg->name);
			      fprintf (output,"         or the number of values > NUM_VAL in the descriptor file\n");
         
            pass = 0;
			      }
           /*        
            return Status */
		return SUCCESS;
		}
            /*        
            return Status */
   return SUCCESS;

}
/* end of CheckNumVal */

/*****************************************************************************
 * 
 * NAME: CheckValType( utility)
 *
 * DESCRIPTION: checks the value type parameter
 *
 * PARAMETERS:  
 *     MyAgg -- (AGGREGATE *) pointer to 1st Aggregate to be checked.
 *     RefAgg -- (AGGREGATE *) pointer to the referance Aggregate to be checked against.
 *     output    (FILE *)      pointer to the output file.                 
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the parameter passed.
 *          FAILURE (int)   the parameter failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/03/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

int CheckValType(AGGREGATE MyAgg, AGGREGATE  RefAgg, FILE *output)
{

 enum {null, integer , real, symbol,string , date , time , 
	 date_time }types;
 unsigned int len1;
 VALUE  refParVal, myParVal= NULL;
 VALUE  mynextVal = NULL;
 PARAMETER refParmVT = NULL;
 PARAMETER myParmV, refParmNV = NULL;
 VALUE_TYPE myType = 0;
 int pass = 1;
 char *typeCheck; 
 
 /* */
    myParmV= FindParameter (MyAgg,"VALUE");
    if(myParmV==NULL ){
      fprintf (output,"WARNING  :: The Aggeragate  = %s has no values to check\n", RefAgg->name);
      pass = 0;
    }
    else
    {

	 refParmVT= FindParameter (RefAgg,"TYPE");
   refParVal= FirstValue(refParmVT);

   if (refParmVT==NULL) {
     fprintf (output,"WARNING  :: The Aggeragate  = %s  type was not specified \n", RefAgg->name);
		pass = 0;
            }
   else if (refParVal->item.value.string == NULL) {
     fprintf (output,"ERROR  :: The Aggeragate  = %s  type field should be within \" \" \n", RefAgg->name);
		pass = 0;
            }
   else
     {
		
 		len1= strlen(refParVal->item.value.string);
		typeCheck = (char *) malloc ((len1 + 1) * sizeof (char));
		strcpy(typeCheck,refParVal->item.value.string);
		/* determine the number of values need to be checked
    */
		refParmNV= FindParameter (RefAgg,"NUM_VAL");
		 
		NumVal = myParmV->value_count;
		
    if( NumVal == 0)
			pass = 0;
    else {
		/* get the first value from my Aggragate 
      */
		    myParVal = FirstValue(myParmV);
        mynextVal = myParVal; 
       }
/*   verify each value in the value field         
  */  
    while (NumVal != 0)
    {
            NumVal--;

           
            myType = mynextVal->item.type;
			
			if (strToUpStrcmp (typeCheck , "INTEGER")==0)
				types = integer;
            else if (strToUpStrcmp (typeCheck , "REAL")==0)
				types = real;
            else if (strToUpStrcmp (typeCheck , "DOUBLE")==0)
				types = real;
			      else if (strToUpStrcmp (typeCheck , "SYMBOL")==0)
				types = symbol;
			      else if (strToUpStrcmp (typeCheck , "STRING")==0)
				types = string;
			      else if (strToUpStrcmp (typeCheck , "TIME")==0)
				types = time;
			      else if (strToUpStrcmp (typeCheck , "DATE")==0)
				types = date;
			      else if (strToUpStrcmp (typeCheck , "DATE_TIME")==0)
				types = date_time;
			      else if (strToUpStrcmp (typeCheck , "DATETIME")==0)
				types = date_time;
			else
				types = null;


      switch (types)
        {
		      case integer:
              
              if(myType != TV_INTEGER)
              {
               
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case real:
              if(myType != TV_REAL)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case symbol:
              if(myType != TV_SYMBOL)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case string:
              if(myType != TV_STRING)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case date:
              if(myType != TV_DATE)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case time:
              if(myType != TV_TIME)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;

          case date_time:
              if(myType != TV_DATE_TIME)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break;
    
		      case null:
              if(myType != TV_NULL)
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              else
              {
              fprintf (output,"ERROR :: The Aggeragate  = %s  in the .met file\n ", RefAgg->name); 
              fprintf (output,"         does not match with the required type %s (try removing \" \") \n",typeCheck);
              pass = 0;
              }
              break; 
            
            }   /* end switch*/

	    mynextVal = NextValue(mynextVal);      
			}         /* end while */

		free (typeCheck);
	
          }
                   
     } 

/*  retrun the status to the calling function  */

  if (pass)
     return SUCCESS;
   else
     return FAILURE;   

}

/* end of CheckValType */


/*****************************************************************************
 * 
 * NAME: CheckValue(utility)
 *
 * DESCRIPTION: checks the Value parameter, when it has a constant value
 *
 * PARAMETERS:  
 *     MyAgg -- (AGGREGATE *) pointer to 1st Aggregate to be checked.
 *     RefAgg -- (AGGREGATE *) pointer to the referance Aggregate to be checked against.
 *     output    (FILE *)      pointer to the output file.                 
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the parameter passed.
 *          FAILURE (int)   the parameter failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/06/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

int CheckValue(AGGREGATE MyAgg, AGGREGATE  RefAgg, FILE *output)
{
 int cValue;
 VALUE  refParVal, myParVal= NULL;
 PARAMETER refParmV = NULL;
 PARAMETER myParmV= NULL;
 VALUE_TYPE myType ;	
 int pass =1 ;
  /* this is to check in case the value is constant */
	refParmV= FindParameter (RefAgg,"VALUE");
     if(refParmV==NULL)
     		return SUCCESS;
     else
     {
		  refParVal= FirstValue(refParmV);
      myType = refParVal->item.type;
		  myParmV= FindParameter (MyAgg,"VALUE");
		  myParVal= FirstValue(myParmV);

      switch (myType)
        {
		  case TV_INTEGER:
              
              
             if (!CheckInteger(refParVal, myParVal))
             {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
                pass = 0;
              }
              break;

          case TV_REAL:

             if (!CheckReal(refParVal, myParVal))
             {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
                pass = 0;
              }
              break;

          case TV_SYMBOL:

 		          if( !CheckSymbol(refParVal, myParVal) )
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
               pass = 0;
              }
            break;

          case TV_STRING:

		          if(!CheckString(refParVal, myParVal))
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
              
                pass = 0;
              }
            break;

          case TV_DATE:

              if(!CheckDateTime(refParVal, myParVal))
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
               
               pass = 0;
              }
            break;

          case TV_TIME:
              
		          if(!CheckDateTime(refParVal, myParVal))
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
              pass = 0;
              }
            break;

          case TV_DATE_TIME:

		          if(!CheckDateTime(refParVal, myParVal))
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
              pass = 0;
              }
            break;
    
		      case TV_NULL:

              cValue =strToUpStrcmp (refParVal->item.value.string, myParVal->item.value.string);
		          if(cValue != 0)
              {     
               fprintf (output,"ERROR :: The Aggeragate  = %s  has a value that does not match,\n ",
                                RefAgg->name);
               fprintf (output,"          with the value in the descriptor file  \n");
              pass = 0;
              }
            break; 
            
            }/* end switch*/

		  
     }
/* return status to calling function for debug */
  if (pass)
     return SUCCESS;
   else
     return FAILURE;
}

/* end of Check Value */

/*****************************************************************************
 * 
 * NAME: CheckInteger( utility)
 *
 * DESCRIPTION: checks the value (Integer) of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *                      
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckInteger(VALUE refIntValue, VALUE myIntValue)
{

   ODLUNITS refUnits = NULL;
   ODLUNITS myUnits = NULL;

   int pass = 1;
   
/* check the number section  */
   if(refIntValue->item.value.integer.number != myIntValue->item.value.integer.number)
        pass = 0;
      
   refUnits = refIntValue->item.value.integer.units;
   
   myUnits = myIntValue->item.value.integer.units;

/*  make sure  we have units on both sides  */
/*  and if one have units while the other doesn't then fail */
   if(  ( (refUnits != NULL) && (myUnits == NULL) ) || 
      ( (refUnits == NULL) && (myUnits != NULL) )  )
     pass = 0;

/*  check the units section of an intetger */
   if( (refUnits != NULL) && (myUnits != NULL) ) 
   {
   if( !CheckOdlUnits(refUnits, myUnits) )
      pass = 0;
   }


/* return status to calling function */
   if (pass)
     return SUCCESS;
   else
     return FAILURE;
 }

 /* end of CheckInteger */



 /*****************************************************************************
 * 
 * NAME: CheckReal( utility)
 *
 * DESCRIPTION: checks the value (Real) of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *                     
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckReal(VALUE refRealValue, VALUE myRealValue)
{

   ODLUNITS refUnits = NULL;
   ODLUNITS myUnits = NULL;

   int pass = 1;
   
/* check the number section of a Real */
   if(refRealValue->item.value.real.number != myRealValue->item.value.real.number)
        pass = 0;
      
   refUnits = refRealValue->item.value.real.units;
   
   myUnits = myRealValue->item.value.real.units;

/*  make sure  we have units on both sides  */
/*  and if one have units while the other doesn't then fail */
   if(  ( (refUnits != NULL) && (myUnits == NULL) ) || 
      ( (refUnits == NULL) && (myUnits != NULL) )  )
     pass = 0;

/*  check the units section of a Real */
   if( (refUnits != NULL) && (myUnits != NULL) ) 
   {
   if( !CheckOdlUnits(refUnits, myUnits) )
      pass = 0;
   }

/* retrun the status to the calling function */
   if (pass)
     return SUCCESS;
   else
     return FAILURE;

 }

 /* end of CheckReal */


/*****************************************************************************
 * 
 * NAME: CheckString( utility)
 *
 * DESCRIPTION: checks the value (String) of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *                      
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckString(VALUE refValue, VALUE myValue)
{

   int pass = 1;

/*  make sure  we have strings on both sides  */
   if ( (refValue->item.value.string != NULL) && (myValue->item.value.string != NULL) )
   {
/* check if the two strings are equal */

      if (strToUpStrcmp (refValue->item.value.string, myValue->item.value.string) != 0)
        pass = 0;
   }
   else
     pass = 0;

/* retrun the status to the calling function */
   if (pass)
     return SUCCESS;
   else
     return FAILURE;


 }

 /* end of CheckString */


/*****************************************************************************
 * 
 * NAME: CheckSymbol( utility)
 *
 * DESCRIPTION: checks the value (Symbol) of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *                      
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckSymbol(VALUE refValue, VALUE myValue)
{

   int pass = 1;

/*  make sure  we have symbols on both sides  */
   if ( (refValue->item.value.string != NULL) && (myValue->item.value.string != NULL) )
   {
/* check if the two symbols are equal */

      if (strToUpStrcmp (refValue->item.value.string, myValue->item.value.string) != 0)
        pass = 0;
   }
   else
     pass = 0;

/* retrun the status to the calling function */
   if (pass)
     return SUCCESS;
   else
     return FAILURE;


 }

 /* end of CheckSymbol */


 /*****************************************************************************
 * 
 * NAME: CheckDateTime( utility)
 *
 * DESCRIPTION: checks the value (Date_Time) of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *     output    (FILE *)      pointer to the output file.                 
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckDateTime(VALUE refValue, VALUE myValue)
{

   int pass = 1;


/* check if we have two values of (year number) fields and they are equal */

     if ( refValue->item.value.date_time.year != myValue->item.value.date_time.year)
            pass = 0;

/* check if we have two values of (day of year) fields and they are equal */

     if ( refValue->item.value.date_time.doy != myValue->item.value.date_time.doy )
            pass = 0;

/* check if we have two values of (month of year) fields and they are equal 

     if ( strToUpStrcmp(refValue->item.value.date_time.month, 
                      myValue->item.value.date_time.month) != 0)*/
     if (refValue->item.value.date_time.month != myValue->item.value.date_time.month)
            pass = 0;

/* check if we have two values of (day of month) fields and they are equal */

     if ( refValue->item.value.date_time.day != myValue->item.value.date_time.day)
            pass = 0;

/* check if we have two values of (Zone hours) fields and they are equal */

     if ( refValue->item.value.date_time.zone_hours != myValue->item.value.date_time.zone_hours)
            pass = 0;

/* check if we have two values of (Zone minutes) fields and they are equal */

     if ( refValue->item.value.date_time.zone_minutes != myValue->item.value.date_time.zone_minutes)
            pass = 0;

/* check if we have two values of (hours of day) fields and they are equal */

     if ( refValue->item.value.date_time.hours != myValue->item.value.date_time.hours)
            pass = 0;

/* check if we have two values of (minutes of hour) fields and they are equal */

     if (refValue->item.value.date_time.minutes != myValue->item.value.date_time.minutes)
            pass = 0;

/* check if we have two values of (seconds of minute) fields and they are equal */

     if (refValue->item.value.date_time.seconds != myValue->item.value.date_time.seconds)
            pass = 0;

 /* check if we have two values of (Nanoseconds) fields and they are equal */

     if ( refValue->item.value.date_time.nanoseconds != myValue->item.value.date_time.nanoseconds)
            pass = 0;

/* retrun the status to the calling function */
     if (pass)
       return SUCCESS;
     else
       return FAILURE;


 }

 /* end of CheckDateTime */

 /*****************************************************************************
 * 
 * NAME: CheckOdlUnits( utility)
 *
 * DESCRIPTION: checks the value's units of a parameter
 *
 * PARAMETERS:  
 *     refValue -- (VALUE *) pointer to the referance value.
 *     myValue  -- (VALUE *) pointer to the value to be checked.
 *                
 *     
 *                      
 *
 * RETURN VALUE: 
 *          SUCCESS (int)   the value passed.
 *          FAILURE (int)   the value failed.
 *     
 * GLOBAL VARIABLES:
 *      
 *                   None
 *	
 * REVISION HISTORY:
 *   08/10/00  Amer Rahabi (original)            
 *
 ****************************************************************************/

 int CheckOdlUnits(ODLUNITS refUnits, ODLUNITS myUnits)
{
   int pass = 1;

   if( (refUnits != NULL) && (myUnits !=NULL) )
   {
/* check  if the uits names are the same */
      if( (refUnits->designator!= NULL ) && (myUnits->designator!= NULL) )
        if ( strToUpStrcmp(refUnits->designator,myUnits->designator) != 0)
            pass = 0;
/* check if the exponents are equal */
      if (refUnits->exponent != myUnits->exponent)
          pass = 0;
   }

/* return the status to the calling function */
          
   if (pass)    
       return SUCCESS;
   else
       return FAILURE;
   

 }
 /* end of CheckOdlUnits */


