/* FILE: odfFieldList.c
 *
 * PROGRAM: ECS ODL field descriptor builder and reader
 *
 * DESCRIPTION
 *     These functions build and access ECS field descriptions. These
 *     descriptions are identified by an Object cantaining parameters
 *     of DATA_LOCATION, MANDATORY, TYPE and NUM_VAL or MAXOCCURRENCES.
 *
 *   
 *
 * buildFieldList - For each main group for in the inventory descriptor file
 *                  the, builds a lstList of the following data stucture
 *
 *    struct groupList
 *    {
 *	    char *groupPath;        ODL Path to the Group
 *	    char *groupType;        ODL Aggregate types
 *	    int groupMandatory;     TRUE if Group is Mandatory
 *	    int groupRepeat;        TRUE of Group is a container
 *	    lstLIST groupFields;    Pointer to fields within the Group
 *     };
 *  
 *
 *                  for each field description within a group, builds a lstList  
 *                  of the following data structure:
 *        struct descList
 *      {
 *        char *fieldName;      Name of the Field Object
 *        int  fieldMandatory;  TRUE if field is mandatory
 *        int  fieldRepeat;     TRUE if field repeats
 *        char *fieldPath;      ODL PATH
 *        char *pathType;       ODL Aggregate types
 *       };
 *
 *
 * fieldDesc - Determines if an aggregate is a field descriptor.
 *
 * buildOdlPath - For a given aggregate, builds the access ODL path and aggregate 
 *                 types 
 *             
 * printFldList - Prints the descList data structure (used for debug).
 *
 *
 *
 * REVISION HISTORY:
 *     08/22/00  Amer Rahabi
 *                moved checkPSA to checkI.c
 *                added external varible CONTROLS
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "odldef.h"
#include "odlinter.h"
#include "lstList.h"
#include "strUtilities.h"
#include "odlFieldList.h"
#include "checkI.h"
#include "metcheck.h"



  /* external varibles */
extern metArgStructure CONTROLS;     /* metcheck controls */
/*******************************/

int buildFieldList(AGGREGATE mainAgg, lstLIST grpList)

{


  AGGREGATE mainSibAgg = NULL;
  AGGREGATE stopAgg = NULL;
  AGGREGATE curAgg = NULL;
  AGGREGATE grpAgg = NULL;

  PARAMETER pParm;
  VALUE     pValue;

  int firstField;
  int fieldCount;
  char pathName[2000],aggType[100],fieldName[200];
  descList fldEntry; 
  groupList grpEntry;
  lstLIST fldList;

  
  stopAgg = mainAgg->parent; /* dont look pass the input Aggregates Parent*/

 
  /*   for each sibling find all field name descriptions */
  firstField = TRUE;
  fieldCount = 0;
  mainSibAgg = NextAggregate(mainAgg);
  while(mainSibAgg) {
      curAgg = NextAggregate(mainSibAgg);
	  
      while(curAgg) { 
    	  if(fieldDesc(curAgg)){	/* we have a field descriptor */  

			  if(firstField) { /* store Group information */
                  grpEntry = malloc(sizeof(struct groupList));
    			  grpAgg = curAgg->parent;
  
				  grpEntry->groupRepeat = FALSE;
                  pParm = FindParameter(grpAgg,"CLASS");
                  if(pParm) {
                      pValue = FirstValue(pParm);
                      if(strcmp(pValue->item.value.string,"M") ==0)
		                  grpEntry->groupRepeat = TRUE;
				  } 

                  if(grpEntry->groupRepeat == FALSE) 
					  grpAgg = mainSibAgg;

				  buildOdlPath(grpAgg,stopAgg,pathName,aggType); 
                  grpEntry->groupPath = malloc(strlen(pathName)+1);     
                  strcpy(grpEntry->groupPath,pathName);
                  grpEntry->groupType = malloc(strlen(aggType)+1);     
                  strcpy(grpEntry->groupType,aggType);
 
				  grpEntry->groupMandatory = TRUE;
		          pParm = FindParameter(grpAgg,"MANDATORY");
                  if(pParm) {
                    pValue = FirstValue(pParm);
                    if(strcmp(pValue->item.value.string,"FALSE") == 0)
				        grpEntry->groupMandatory = FALSE;
				  }
      
          
 
				  
				  fldList = lstInitList();
                  grpEntry->groupFields = fldList;
                  lstAddLast(grpList, grpEntry); /* Add to List */			  
                  firstField = FALSE;
				  
			  }
			  
             /* Build field desc. data structure */
              ++fieldCount;
			  strcpy(fieldName,curAgg->name);
              buildOdlPath(curAgg,grpAgg->parent,pathName,aggType); 
              fldEntry = malloc(sizeof(struct descList));
		      fldEntry->fieldName = malloc(strlen(fieldName)+1);     
              strcpy(fldEntry->fieldName,fieldName);
              fldEntry->fieldPath = malloc(strlen(pathName)+1);     
              strcpy(fldEntry->fieldPath,pathName);
              fldEntry->pathType = malloc(strlen(aggType)+1);     
              strcpy(fldEntry->pathType,aggType);

              fldEntry->fieldMandatory = FALSE;
              pParm = FindParameter(curAgg,"MANDATORY");
              if(pParm) {
                  pValue = FirstValue(pParm);
                  if(strcmp(pValue->item.value.string,"TRUE") ==0)
				      fldEntry->fieldMandatory = TRUE;
			  }
      
          
              fldEntry->fieldRepeat = FALSE;
              pParm = FindParameter(curAgg->parent,"CLASS");
              if(pParm) {
              pValue = FirstValue(pParm);
                  if(strcmp(pValue->item.value.string,"M") ==0)
		              fldEntry->fieldRepeat = TRUE;
			  } 


              lstAddLast(fldList, fldEntry); /* Add to List */
 

		  }/* End if(fieldDesc(curAgg))*/

          curAgg = NextAggregate(curAgg);
          if ( mainSibAgg->right_sibling == curAgg)                 
        	  curAgg = NULL;
          else if(curAgg->parent == stopAgg)
          curAgg = NULL; 
      }/* end while curAgg */
  
      mainSibAgg = mainSibAgg->right_sibling; /* Process next sibling */ 
      firstField = TRUE;
  } /*end  while(mainSibAgg) */

  return fieldCount;

}
/*************************************************************/
int fieldDesc(AGGREGATE agg)
{
  
  if(!FindParameter(agg,"TYPE"))
      return FALSE;

  if(!FindParameter(agg,"DATA_LOCATION"))
	  return FALSE;

  if(!FindParameter(agg,"MANDATORY"))
	  return FALSE;

  if(FindParameter(agg,"NUM_VAL"))
	  return TRUE;
	   
  if(FindParameter(agg,"MAXOCCURRENCES"))
	  return TRUE;

  return FALSE;
}
/************************************************************/
void buildOdlPath(AGGREGATE start, AGGREGATE stop,char *pathName,
                  char *aggType)

{
   AGGREGATE nextS;
   char tmpName[200];
   char tmpType[200];
   char *r;
   int i;
   nextS = start;
  
   pathName[0] = '\0';
   tmpType[0] = '\0';
   while(nextS->parent != stop) {
       sprintf(tmpType,"%s%d",tmpType,nextS->kind);
       strcpy(tmpName,pathName);
       strcpy(pathName,nextS->name);
       if(strlen(tmpName) >0){
           strcat(pathName,"/");
           strcat(pathName,tmpName);
       }
       nextS = nextS->parent;
    
   }

   r = tmpType + strlen(tmpType)-1;  
   for(i = 0;r >=  tmpType; r--) 
        aggType[i++] = *r;
   aggType[strlen(tmpType)] = '\0';     
 
}
/**********************************************************/
void printFldList(lstLIST grpList)
{

  descList descEntry;
  groupList grpEntry;

  lstLIST fldList;
  lstStopIteration(grpList); 
   grpEntry = (groupList) lstIterator(grpList);
   while (grpEntry != NULL){
      printf("\nGroup = %s %s m=%d r=%d\n",
      grpEntry->groupPath,grpEntry->groupType,
      grpEntry->groupMandatory,grpEntry->groupRepeat);
      fldList = grpEntry->groupFields;
      lstStopIteration(fldList); 
      descEntry = (descList) lstIterator(fldList);
      while (descEntry != NULL){
          printf("Name = %s Man = %d Rep = %d path =%s\n",
          descEntry->fieldName,descEntry->fieldRepeat,
          descEntry->fieldMandatory,descEntry->pathType);
          printf("Path = %s\n\n",descEntry->fieldPath);
          descEntry = (descList) lstIterator(fldList);
	  }   

	   grpEntry = (groupList) lstIterator(grpList);
   }
}  



